import logging
from typing import Union

import requests

from cbutil.util import remove_newline

logger = logging.getLogger(__file__)

Project_id = Union[str, int]


def url_encode(project_id: Project_id) -> str:
    if isinstance(project_id, str):
        return project_id.replace("/", "%2F")
    else:
        return project_id


def add_pagination(url: str, page, *, per_page=20) -> str:
    return f"{url}?per_page={per_page}&page={page}"


def get_api_url(instance_url: str, project_id: Project_id) -> str:
    return f"{instance_url}/api/v4/projects/{url_encode(project_id)}"


def get_api_url_commits(base_api_url: str) -> str:
    return f"{base_api_url}/repository/commits"


def get_last_commit(instance_url: str, project_id: Project_id, branch: str = ''):
    url = get_api_url_commits(get_api_url(instance_url, project_id))
    url = add_pagination(url, page=1, per_page=1)
    params = {'ref_name': branch if branch else 'master'}
    commit = requests.get(url, params=params)
    logger.info(f"requesting {url} with params")
    if commit.status_code != 200:
        commit.raise_for_status()
    return commit.json()[0]


def get_commit_infos_from_api(commit: dict) -> dict:
    return commit['id'], remove_newline(commit['message'])


def get_git_infos_from_api(instance_url, project_id: Project_id,
                           *,
                           branch='main',
                           commit_key="commit",
                           commit_msg_key="commit_message"):
    last_commit = get_last_commit(instance_url, project_id, branch=branch)
    commit, commit_msg = get_commit_infos_from_api(last_commit)
    commit_msg = remove_newline(commit_msg)
    return {commit_key: commit, commit_msg_key: commit_msg}
