import grafanalib.core
from grafanalib.influxdb import InfluxDBTarget

from .panels import get_arguments


def get_target_class_name(name, *, prefix="Wrapped"):
    if not name.startswith(prefix):
        raise ValueError(f"{name} does not start with {prefix}")
    return name.replace(prefix, "")


def get_grafanalib_object(wrapped_object):
    cls_name = get_target_class_name(wrapped_object.__class__.__name__)
    if cls_name == "InfluxDBTarget":
        target_class = InfluxDBTarget
    else:
        target_class = getattr(grafanalib.core, cls_name)
    kwargs = get_arguments(wrapped_object)
    for kwarg, value in kwargs.items():
        if value.__class__.__name__.startswith("Wrapped"):
            kwargs[kwarg] = get_grafanalib_object(value)
        elif isinstance(value, list):
            if value and value[0].__class__.__name__.startswith("Wrapped"):
                kwargs[kwarg] = [get_grafanalib_object(o) for o in value]

    return target_class(**kwargs)
