from dataclasses import dataclass, field, fields
from typing import List


@dataclass
class WrappedTarget:
    query: str
    alias: str = None


class WrappedInfluxDBTarget(WrappedTarget):
    pass


@dataclass
class WrappedRepeat:
    direction: str
    variable: str
    maxPerRow: int = None


@dataclass
class WrappedPanel:
    title: str
    dataSource: str
    targets: List[WrappedTarget] = field(default_factory=list)
    thresholdType: str = 'absolute'
    thresholds: List = field(default_factory=list)
    extra_kwargs: dict = field(default_factory=dict)


@dataclass
class WrappedTimeSeries(WrappedPanel):
    unit: str = 'none'
    axisLabel: str = ''
    pointSize: int = 1
    repeat: WrappedRepeat = None


@dataclass
class WrappedStat(WrappedPanel):
    alignment: str = 'center'
    colorMode: str = 'value'
    graphMode: str = 'area'
    reduceCalc: str = 'last'
    orientation: str = 'auto'
    transparent: bool = True


@dataclass
class WrappedRowPanel:
    title: str
    collapsed: bool = True
    panels: List[WrappedPanel] = field(default_factory=list)


@dataclass
class WrappedDashboardList(WrappedPanel):
    searchTags: List[str] = field(default_factory=list)
    showHeadings: bool = False
    showSearch: bool = True
    showStarred: bool = False


@dataclass
class WrappedTime:
    start: str
    end: str


@dataclass
class WrappedDashboard:
    title: str
    description: str
    tags: List[str] = field(default_factory=list)
    timezone: str = "browser"
    rows: List = field(default_factory=list)
    panels: List = field(default_factory=list)
    templating = None
    annotation = None
    time: WrappedTime = field(default_factory=lambda: WrappedTime(start='now-30d', end='now'))

    def __post_init__(self):
        if len(self.rows) != 0 and len(self.panels) != 0:
            raise ValueError("Either rows or panels")


@dataclass
class WrappedTemplate:
    name: str
    query: str
    type: str = "query"
    includeAll: bool = True
    multi: bool = True
    regex: str = None


def get_arguments(wrapped) -> dict:
    ret = dict()
    for cur_field in fields(wrapped):
        if cur_field.type == dict:
            ret.update(getattr(wrapped, cur_field.name))
        elif value := getattr(wrapped, cur_field.name):
            ret.setdefault(cur_field.name, value)
    return ret
