from unittest.mock import patch

from dashboards.grafanalib_backend import (get_grafanalib_object,
                                           get_target_class_name)
from dashboards.panels import (WrappedDashboard, WrappedInfluxDBTarget,
                               WrappedPanel, WrappedTime, WrappedTimeSeries,
                               get_arguments)


def test_get_target_class_Panel():
    assert get_target_class_name("WrappedPanel") == "Panel"
    assert get_target_class_name("PrefixPanel", prefix="Prefix") == "Panel"
    panel = WrappedPanel("test", "InfluxDB-1")
    assert get_target_class_name(panel.__class__.__name__)


@patch('dashboards.grafanalib_backend.grafanalib.core')
def test_get_grafanalib_panel(mock_grafanalib):
    panel = WrappedPanel("test", "InfluxDB-1")
    kwargs = get_arguments(panel)

    get_grafanalib_object(panel)
    mock_grafanalib.Panel.assert_called_with(**kwargs)


@patch('grafanalib.core')
def test_get_grafanalib_ts(mock_grafanalib):
    panel = WrappedTimeSeries("test", "InfluxDB-1")
    kwargs = get_arguments(panel)
    get_grafanalib_object(panel)
    mock_grafanalib.TimeSeries.assert_called_with(**kwargs)


@patch('dashboards.grafanalib_backend.InfluxDBTarget')
@patch('grafanalib.core')
def test_get_grafanalib_ts_with_target(mock_grafanalib_core, mock_grafanalib_influx):
    target = WrappedInfluxDBTarget('SELECT "mlupsperprocess" FROM "UniformGridCPU"')
    panel = WrappedTimeSeries("test", "InfluxDB-1", targets=[target])
    get_grafanalib_object(panel)
    mock_grafanalib_core.TimeSeries.assert_called()
    mock_grafanalib_influx.assert_called_with(**get_arguments(target))


@patch('grafanalib.core')
def test_time(mock_grafanalib):
    time = WrappedTime('now-30d', 'now')
    get_grafanalib_object(time)
    mock_grafanalib.Time.assert_called_with(start='now-30d', end='now')


@patch('grafanalib.core')
def test_dashboard(mock_grafanalib):
    dashboard = WrappedDashboard("title", "description")
    get_grafanalib_object(dashboard)
    mock_grafanalib.Time.assert_called()
    mock_grafanalib.Dashboard.assert_called()
