from cbutil.postprocessing import process_linewise
from cbutil.processing_functions import mesa_pd_text
from cbutil.util import read_file_line_wise


def id(x):
    return x


def test_linewise_trivial():
    lines = ["aaa", "bbb"]
    for raw, processed in zip(lines, process_linewise(id, lines)):
        assert raw == processed


def test_linewise_split():
    def splitter(line):
        splitted = line.split(":")
        return splitted[0], splitted[1]

    lines = ["lhs1:rhs1", "lhs1:rhs1"]

    for raw, processed in zip(lines, process_linewise(splitter, lines)):
        assert splitter(raw) == processed


def test_mesa_pd():
    mesa_pd_example_output = "tests/mesa_pd.txt"
    expected = {'AssocToBlock': 6.43,
                'ContactDetection': 113.55,
                'Euler': 19.65,
                'GenerateLinkedCells': 12.96,
                'ReduceContactHistory': 6.17,
                'ReduceForce': 0.0,
                'SNN': 0.0,
                'SpringDashpot': 60.14,
                'SpringDashpotSpring': 226.38}

    for key, value in process_linewise(mesa_pd_text, read_file_line_wise(mesa_pd_example_output)):
        assert value == expected[key]
